# NeoBoard

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from DateTime.DateTime import DateTime
from NB_Utils import getDTML, setFormError, getReqAttr, setFormMsg, traverseFolder, getNextElementName, isDateTimeValid
from NB_BaseClasses import NB_AdvancedFolder
from NB_Mailer import NB_Mailer
from NeoBoardArticle import NeoBoardArticle
import os

stylesheet = getDTML('themes/default/templates/stylesheet')
manage_addNeoBoardForm_NB =  getDTML('dtml/NeoBoardAdd')

def manage_addNeoBoard_NB(self, 
			id=None, 
			title='', 
			description='', 
			theme=None,
			mailhost=None,
			formats=None,
			charset='',
			language='',			
			effective_date=None,
			expire_date=None,
			archive_path='',
			bad_extensions=[],
			max_file_size=5,
			max_image_width=200,
			max_image_height=200,
			max_list_title_length=30,
			enabled=0,
			read_only=0,
			page_limit=20,
			cacheable_content=0,			
			mail_article=0,
			num_attachments=0,
			allow_empty_body=0,
			acl_user_auth=0,
			allow_empty_password=0,
			allow_empty_email=0,
			auto_approve_article=0,
			REQUEST=None):
	""" Add NeoBoard"""		                            

	if not id:
		return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'id', self.getMessage_NB("Invalid ID!")))

	instance = NeoBoard()
	
	instance.id=id
	instance.title=title
	
	instance.menu_title=instance.getTitle()
	
	instance.description=description

	if num_attachments > 0:
		if not archive_path:
			return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'archive_path', self.getMessage_NB("You must specify the archive path to allow attachments!")))			
		else:
			if not archive_path.endswith(os.sep):
				archive_path = archive_path + os.sep
				
			if os.path.isdir(archive_path):
				if not os.access(archive_path, os.R_OK | os.W_OK):
					return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'archive_path', self.getMessage_NB("You don't have permissions to use that archive path!")))	
			else:
				try:
					os.mkdir(archive_path, 0755)
				except:
					return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'archive_path', self.getMessage_NB("Archive path doesn't exist and cannot be created!")))
					
			instance.archive_path=archive_path
	else:				
		instance.archive_path=archive_path
		
	instance.max_file_size=max_file_size
	instance.max_image_width=max_image_width
	instance.max_image_height=max_image_height
	instance.max_list_title_length=max_list_title_length
	instance.num_attachments=num_attachments
	instance.page_limit = page_limit
	instance.enabled=enabled
	instance.read_only=read_only
	instance.mail_article=mail_article
	instance.allow_empty_body = allow_empty_body
	instance.acl_user_auth= acl_user_auth
	instance.auto_approve_article= auto_approve_article
	
	instance.bad_extensions=bad_extensions
	
	if not acl_user_auth:
		instance.allow_empty_password = allow_empty_password
		instance.allow_empty_email = allow_empty_email
	
	instance.setTheme(theme)

	if mailhost:
		try:    
			dummy=getattr(self, mailhost)
		except: 
			return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'mailhost', self.getMessage_NB("Invalid Mailhost!")))
	else:
			return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'mailhost', self.getMessage_NB("Named Mailhost not found!")))
	
	instance.mailhost = mailhost
	instance.setNextArticleNum(0)
	instance.setTotalNumArticles(0)
	
	if formats: 
		instance.formats=formats
	else:
		instance.formats= ['HTML',]
		
	if charset: 
		instance.charset=charset
	else:
		instance.charset=''
	
	if language:		
		instance.language=language
	else:
		instance.language=''
	
	instance.creation_date = DateTime()
	
	if effective_date:
		if not isDateTimeValid(effective_date):
			return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'effective_date', self.getMessage_NB("DateTime format is invalid!")))		
		instance.effective_date = effective_date
	else:
		instance.effective_date = DateTime()
		
	if expire_date:
		if not isDateTimeValid(expire_date):
			return self.manage_addNeoBoardForm_NB(setFormError(REQUEST, 'expire_date', self.getMessage_NB("DateTime format is invalid!")))	
	
		instance.expire_date = expire_date
	else:
		instance.expire_date = None

	instance.setContentCacheable(cacheable_content)
	instance.posting_categories = {}
	instance.managers = {}
	
	instance._upgraded_up_to = NeoBoard.__NB_VERSION__
	
	self._setObject(id, instance)
	self._getOb(id)._addCatalog()	
		
	if REQUEST is not None:
		return self.manage_main(self, REQUEST, update_menu=1)	

class NeoBoard(NB_AdvancedFolder):
	"""
		NeoBoard Class
	"""
	
	__NB_VERSION__  = '1.0b'
	
	meta_type='NeoBoard'
	icon='misc_/NeoBoard/neoboard_icon'     

	meta_types=(
		{'name': 'NeoBoard Article', 'action':'addNeoBoardArticleForm', 'permission': 'Add NeoBoard Article'},
		{'name': 'NeoBoard Notice', 'action':'addNeoBoardArticleForm', 'permission': 'Add NeoBoard Article'},
	)
	all_meta_types = meta_types

	manage_options=(
		{'label':'View',       'action':'index_html'},
		{'label':'Properties', 'action':'manage_editForm'},
		{'label':'Categories', 'action':'manage_editCategoriesForm'},
		{'label':'Managers',   'action':'manage_editManagersForm'},
		{'label':'Security',   'action':'manage_access'},
		{'label':'Ownership',  'action':'manage_owner'},
	)

	_renderables = ('RAW', 'HTML','STX', 'DTML')
	_max_num_attachments = 5

	security = ClassSecurityInfo()	
	security.declareObjectProtected('View')	

	def post(self, REQUEST=None):
		"""
			Returns Post Template
		"""
		if REQUEST is not None:		
			return apply(self.getThemeObject_NB(id='NeoBoardArticleAdd', obj_type='templates', render=0), (self, self.REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardArticleAdd', obj_type='templates', render=0), (self, REQUEST))
				
	def neoboard_header(self, REQUEST=None):
		"""
			Returns Header Template
		"""		
		if REQUEST is not None:				
			return apply(self.getThemeObject_NB(id='NeoBoardHeader', obj_type='templates', render=0), (self, REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardHeader', obj_type='templates', render=0), (self, self.REQUEST))
			
	def neoboard_footer(self, REQUEST=None):
		"""
			Returns Footer Template
		"""		
		if REQUEST is not None:					
			return apply(self.getThemeObject_NB(id='NeoBoardFooter', obj_type='templates', render=0), (self, REQUEST))
		else:
			return apply(self.getThemeObject_NB(id='NeoBoardFooter', obj_type='templates', render=0), (self, self.REQUEST))		

	security.declareProtected('Edit NeoBoard', 'manage_editForm')
	manage_editForm = getDTML('dtml/NeoBoardEdit')

	security.declareProtected('Edit NeoBoard', 'manage_editCategoriesForm')
	manage_editCategoriesForm = getDTML('dtml/NeoBoardCategoriesEdit')
	
	security.declareProtected('Edit NeoBoard', 'manage_editManagersForm')
	manage_editManagersForm = getDTML('dtml/NeoBoardManagersEdit')	

	security.declareProtected('Edit NeoBoard', 'manage_editNeoBoard')
	def manage_editNeoBoard(self, 
			title='', 
			description='', 
			theme=None,
			mailhost='',
			archive_path='',
			bad_extensions=[],
			max_file_size=5,
			max_image_width=200,
			max_image_height=200,
			max_list_title_length=30,
			priority=100,
			page_limit=20,
			formats=None,
			charset='',
			language='',				
			effective_date=None,
			expire_date=None,
			cacheable_content=0,			
			enabled=0,
			read_only=0,
			mail_article=0,
			acl_user_auth=0,
			num_attachments=0,
			allow_empty_body=0,
			allow_empty_password=0,
			allow_empty_email=0,
			auto_approve_article=0,
			recreate_catalog=0,
			REQUEST=None):
		""" Edit NeoBoard"""		                            
	
		
		self.title=title		
		self.menu_title=self.getTitle()

		self.description=description
		
		if num_attachments > 0:
			if not archive_path:
				return self.manage_editForm(setFormError(REQUEST, 'archive_path', self.getMessage_NB("You must specify the archive path to allow attachments!")))			
			else:
				if not archive_path.endswith(os.sep):
					archive_path = archive_path + os.sep
					
				if os.path.isdir(archive_path):
					if not os.access(archive_path, os.R_OK | os.W_OK):
						return self.manage_editForm(setFormError(REQUEST, 'archive_path', self.getMessage_NB("You don't have permissions to use that archive path!")))	
				else:
					try:
						os.mkdir(archive_path, 0755)
					except:
						return self.manage_editForm(setFormError(REQUEST, 'archive_path', self.getMessage_NB("Archive path doesn't exist and cannot be created!")))
						
			self.archive_path=archive_path
		
		self.bad_extensions=bad_extensions
		self.num_attachments=num_attachments
		self.page_limit = page_limit
		self.enabled=enabled
		self.read_only=read_only
	
		if not self.getTotalNumArticles():
			self.acl_user_auth = acl_user_auth
			
		self.max_file_size = max_file_size
		self.max_image_width=max_image_width
		self.max_image_height=max_image_height
		self.max_list_title_length=max_list_title_length
		self.mail_article=mail_article
		self.allow_empty_body = allow_empty_body
		self.auto_approve_article = auto_approve_article

		if not self.useACLUserAuth():
			self.allow_empty_password = allow_empty_password
			self.allow_empty_email = allow_empty_email
	
			
		self.setTheme(theme)			

		if mailhost:
			try:    
				dummy=getattr(self, mailhost)
			except: 
				return self.manage_editForm(setFormError(REQUEST, 'mailhost', self.getMessage_NB("Invalid Mailhost!")))
		else:
				return self.manage_editForm(setFormError(REQUEST, 'mailhost', self.getMessage_NB("Named Mailhost not found!")))
		
		self.mailhost = mailhost
		

		if formats: 
			changed = 0
			for format in formats:
				if format not in self.getFormats():
					changed = 1

			for format in self.getFormats():
				if format not in formats:
					changed = 1
			
			self.formats = formats		
			if changed:
				self._reRenderAll()
			
		else:
			self.formats=[self.getDefaultFormat(),]
			self._reRenderAll()
			
		if charset: 
			self.charset=charset
		else:
			self.charset=self.getDefaultCharset()
	
		if language:		
			self.language=language
		else:
			self.language=self.getDefaultLanguage()
		
		if effective_date:
			if not isDateTimeValid(effective_date):
				return self.manage_addArticleForm(setFormError(REQUEST, 'effective_date', self.getMessage_NB("DateTime format is invalid!")))		
			self.effective_date = effective_date
		else:
			self.effective_date = DateTime()
			
		if expire_date:
			if not isDateTimeValid(expire_date):
				return self.manage_addArticleForm(setFormError(REQUEST, 'expire_date', self.getMessage_NB("DateTime format is invalid!")))	
		
			self.expire_date = expire_date
		else:
			self.expire_date = None

		self.setContentCacheable(cacheable_content)

		if recreate_catalog:
			self._addCatalog()

		if REQUEST is not None:
			return self.manage_editForm(self, REQUEST)

	security.declareProtected('Edit NeoBoard', 'manage_batchAction')
	def manage_batchAction(self,
			batch_items=None,
			submit=None,
			to_board=None,
			REQUEST=None):
		"""
			Batch Action
		"""

		if not self.isNeoBoardManager():
			return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("You are not authorized to perform this action.")))

		if batch_items is None or len(batch_items) <= 0:
			return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("No items selected!")))

		if submit is None or not (submit == 'Delete' or submit == 'Move' or submit == 'Copy' or submit == 'Approve' or submit == 'Disapprove'):
			return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("Invalid submit message!")))


		if submit == 'Move' or submit == 'Copy':

			if to_board is None:
				return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("Receiving board not specified!")))
		
			if not self.aq_parent._getOb(to_board).isNeoBoardManager():
				return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("You are not a manager for the receiving board.")))

			if not self.useACLUserAuth() == self.aq_parent._getOb(to_board).useACLUserAuth():
				return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("The receiving board is using a different authentication scheme.")))

			for article in batch_items:
				if self.aq_parent._getOb(to_board).getNumAttachmentsAllowed() < self._getOb(article).getNumAttachments():
					return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("The receiving board has can't hold all the attachments the article '%s' has.")))

			
			try:
				receiver = self.aq_parent._getOb(to_board)
			except:
				return self.manage_main(REQUEST=setFormError(REQUEST, 'default', self.getMessage_NB("The named receiving board doesn't exist.")))

			if submit == 'Move':
    				pointer = self.manage_cutObjects(batch_items)
    				msg = self.getMessage_NB("%d message(s) moved to '%s' board.") % (len(batch_items), receiver.title)
    			else:
    				pointer = self.manage_copyObjects(batch_items)
    				msg = self.getMessage_NB("%d message(s) copied to '%s' board.") % (len(batch_items), receiver.title)
    				
			receiver.manage_pasteObjects(pointer)
			self._resetNeoBoard()
			receiver._resetNeoBoard()

			REQUEST=setFormMsg(REQUEST, msg)

		elif submit == 'Approve' or submit == 'Disapprove':
			threads = 0
			for item in batch_items:
				try:
					if submit=='Approve':
						self._getOb(item).approveThread()
					else:
						self._getOb(item).disApproveThread()
						
					threads = threads + 1
				except:
					threads = threads - 1
			
			if submit=='Approve':
				REQUEST=setFormMsg(REQUEST, self.getMessage_NB("%d thread(s) approved out of %d thread(s) requested.") % (threads, len(batch_items)))
			else:
				REQUEST=setFormMsg(REQUEST, self.getMessage_NB("%d thread(s) disapproved out of %d thread(s) requested.") % (threads, len(batch_items)))
		else:
			num_messages = len(batch_items)
			self.manage_delObjects(batch_items)
			self._resetNeoBoard()
			REQUEST=setFormMsg(REQUEST, self.getMessage_NB("%d message(s) deleted.") % (num_messages,))
				
		if REQUEST is not None:
			return self.manage_main(REQUEST=REQUEST)

	security.declareProtected('Edit NeoBoard', 'manage_addPostingCategory')
	def manage_addPostingCategory(self, category_id, category_title, REQUEST=None):
		"""
			Adds Posting Category
		"""

		if not category_id:
			return self.manage_editCategoriesForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	

		if not category_title:
			return self.manage_editCategoriesForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	
			
		if hasattr(self, 'posting_categories'):
			self.posting_categories[category_id] = category_title
		else:
			self.posting_categories = { category_id : category_title, }

		self._p_changed = 1

		if REQUEST is not None:
			return self.manage_editCategoriesForm(self, REQUEST)		

	security.declareProtected('Edit NeoBoard', 'manage_editPostingCategory')
	def manage_editPostingCategory(self, category_id, new_category_id, category_title, REQUEST=None):
		"""
			Edits Posting Category
		"""

		if not new_category_id:
			return self.manage_editCategoriesForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	

		if not category_title:
			return self.manage_editCategoriesForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	
			
		if not category_id == new_category_id:
			articles = self.getNeoBoardParentArticles()
			for article in articles:
				if article.getCategory() == category_id:
					article.setCategory(new_category_id)
			
			if self.posting_categories.has_key(category_id):
				del self.posting_categories[category_id]
				self.posting_categories[new_category_id] = category_title				
		else:
			self.posting_categories[category_id] = category_title

		self._p_changed = 1

		if REQUEST is not None:
			return self.manage_editCategoriesForm(self, REQUEST)		

	security.declareProtected('Edit NeoBoard', 'manage_delPostingCategory')
	def manage_delPostingCategory(self, category_id, REQUEST=None):
		"""
			Deletes Posting Category
		"""

		if not category_id:
			return self.manage_editCategoriesForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	

		if self.posting_categories.has_key(category_id):
			del self.posting_categories[category_id]

		self._p_changed = 1

		if REQUEST is not None:
			return self.manage_editCategoriesForm(self, REQUEST)						

	security.declareProtected('Edit NeoBoard', 'manage_addManager')
	def manage_addManager(self, username, email=None, REQUEST=None):
		"""
			Adds Manager
		"""

		if not username:
			return self.manage_editManagersForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	

		user = self.acl_users.getUser(username)
		
		if not user:
			return self.manage_editManagersForm(setFormError(REQUEST, 'default', self.getMessage_NB("No Such User!")))
		
		if not email:
			email = None
			
		if hasattr(self, 'managers'):
			self.managers[username] = email
		else:
			self.managers = { username : email, }

		self._p_changed = 1

		if REQUEST is not None:
			return self.manage_editManagersForm(self, REQUEST)
	
	manage_editManager = manage_addManager

	security.declareProtected('Edit NeoBoard', 'manage_delManager')
	def manage_delManager(self, username, REQUEST=None):
		"""
			Deletes Manager
		"""

		if not username:
			return self.manage_editManagersForm(setFormError(REQUEST, 'default', self.getMessage_NB("The field cannot be empty!")))	

		if self.managers.has_key(username):
			del self.managers[username]

		self._p_changed = 1

		if REQUEST is not None:
			return self.manage_editManagersForm(self, REQUEST)
			
    	def _get_id(self, id):
		"""
			New ID's for moved articles
		"""    	
	
		if id.startswith('a_'):
			new_id = 'a_' + str(self.getNextArticleNum())
		else:
			new_id = 'n_' + str(self.getNextArticleNum(1))
		return new_id
				
	def getManagers(self):
		"""
			Returns Board Managers
		"""		
		return self.managers
		
	def isNeoBoardManager(self):
		"""
			Tells if the user is a manager
		"""		
		if self.REQUEST.AUTHENTICATED_USER.has_role('Manager'):
			return 1
			
		return self.REQUEST.AUTHENTICATED_USER.getUserName() in self.getManagers().keys()
	
	def getNeoBoards(self):
		"""
			Returns a list of NeoBoards on the same level
		"""		
		return self.getParent().objectValues(self.meta_type)
	
	def getNeoBoardNotices(self):
		"""
			Returns a list of NeoBoard Notice Articles
		"""		
		return self.objectValues('NeoBoard Notice')
	
	def getPostingCategories(self):
		"""
			Returns Posting Categories
		"""				
		return self.posting_categories		

	def getCurrentNeoBoard(self):
		"""
			Returns Self
		"""		
		return self.getSelf()

	def getNeoBoardURL(self):
		"""
			Returns Absoulte URL of the current Board
		"""		
		return self.absolute_url()

	def getNeoBoardArticle(self, article):
		"""
			Returns the named Article
		"""		
		if hasattr(self, article):
			return self.article
		return None

	def getNeoBoardParentArticles(self):
		"""
			Returns a list of top-most NeoBoard Articles without replies
		"""		
		return self.objectValues('NeoBoard Article')

	def getNeoBoardArticles(self, skey=None, reverse=0, default=None, default_reverse=0, search_fields='', search_keywords=''):
		"""
			Returns a complete list of NeoBoard Articles with all the replies
		"""
		
		if search_fields and search_keywords:
		
			catalog = self.getCatalog()
			
			if catalog is None:
				return []

			result = []
			if hasattr(search_fields, 'title'):
				for idx in catalog({'title': search_keywords}):
					obj = idx.getObject()
					if obj and obj not in result and obj.meta_type=='NeoBoard Article':
						result.append(obj)
				
			if hasattr(search_fields, 'body'):
				for idx in catalog({'getRenderedContent': search_keywords}):
					obj = idx.getObject()
					if obj and obj not in result and obj.meta_type=='NeoBoard Article':
						result.append(obj)
				
			if hasattr(search_fields, 'author'):
				for idx in catalog({'author': search_keywords}):
					obj = idx.getObject()
					if obj and obj not in result and obj.meta_type=='NeoBoard Article':
						result.append(obj)
				
			if hasattr(search_fields, 'attachments'):
				for idx in catalog({'getAttachmentsSearchableText': search_keywords}):
					obj = idx.getObject()
					if obj and obj not in result and obj.meta_type=='NeoBoard Article':
						result.append(obj)
			
			return result
					
		else:
			return traverseFolder(self, 'NeoBoard Article', skey=skey, reverse=reverse, default=default, default_reverse=default_reverse)

	def getMaxFileSize(self):
		"""
			Returns the Upper Size Limit to Attachments
		"""		
		return self.max_file_size

	def getMaxImageWidth(self):
		"""
			Returns Maximum Image Width to trigger a thumbnail
		"""		
		return self.max_image_width
	
	def getMaxImageHeight(self):
		"""
			Returns Maximum Image Height to trigger a thumbnail
		"""				
		return self.max_image_height

	def checkSize(self, files, extra=0):
		"""
			Tells if the given file is bigger than the size limit
		"""				
		t_size = extra
		for file in files:
			t_size = t_size + int(len(file.read())/1047576)
			file.seek(0)
		
		if t_size > self.getMaxFileSize():
			return 0
		else:
			return 1

	def getMaxListTitleLength(self):
		"""
			Returns maximum length of titles when appearing on article lists
		"""
		return self.max_list_title_length

	def getNextArticleNum(self, notice=0):
		"""
			Returns the next article number to use
		"""		
		new_num = self.next_article_num + 1
		
		if notice:
			num_notices = self.objectValues('NeoBoard Notice')
			return len(num_notices) + 1
		
		if hasattr(self, 'a_' + str(new_num)):
			self._resetNeoBoard()
			new_num = self.next_article_num + 1
		
		return new_num

	def setNextArticleNum(self, num):
		"""
			Sets the next article number to use
		"""		
		
		self.next_article_num = num
	
	def setTotalNumArticles(self, num):
		"""
			Sets the total number of articles this board contains
		"""		
		
		self.total_num_articles = num 

	def getTotalNumArticles(self):
		"""
			Returns the total number of articles this board contains
		"""		
		
		return self.total_num_articles

	def getCurrentArticleURL(self):
		"""
			Returns the relative URL of the current article
		"""		
		
		url = self.REQUEST.URL1
		return url.replace(self.getNeoBoardURL() + '/', '')

	def getArchivePath(self):
		"""
			Returns Archive Path
		"""		
		
		return self.archive_path

	def getBadExtensions(self):
		extensions = []
		
		for ext in self.bad_extensions:
			if not ext.startswith('.'):
				extensions.append('.' + ext)
			else:
				extensions.append(ext)
		return extensions		

	def isMailEnabled(self):
		"""
			Tells if mailing is enabled
		"""		
		
		return self.mail_article

	def isReadOnly(self):
		"""
			Tells if the board is read-only
		"""		
	
		return self.read_only
				
	def attachmentsAllowed(self):
		"""
			Tells if attachments are allowed
		"""		
		if self.getNumAttachmentsAllowed() > 0:
			return 1
		else:
			return 0
	def getNumAttachmentsAllowed(self):
		"""
			Returns the number of attachments allowed
		"""		
		
		num = self.num_attachments
		
		if num > self._max_num_attachments:
			return self._max_num_attachments
		else:
			return num

	def emptyBodyAllowed(self):
		"""
			Tells if empty body is allowed
		"""		
		
		return self.allow_empty_body
	
	def useACLUserAuth(self):
		"""
			Tells if ACL user authentication is in use
		"""		
		
		if hasattr(self, 'acl_user_auth'):
			return self.acl_user_auth
		else:
			return 0
		
	def emptyPasswordAllowed(self):
		"""
			Tells if empty password is allowed
		"""		
		
		if hasattr(self, 'allow_empty_password'):
			return self.allow_empty_password
		else:
			return 0

	def emptyEmailAllowed(self):
		"""
			Tells if empty email address is allowed
		"""		
		
		if hasattr(self, 'allow_empty_email'):
			return self.allow_empty_email
		else:
			return 0
	
	def autoApproveArticle(self):
		if hasattr(self, 'auto_approve_article'):
			return self.auto_approve_article
		else:
			return 0

	def index_html(self, REQUEST={}, RESPONSE=None):
		"""
			Returns index_html
		"""
		
		if not self.isEnabled() and not self.isNeoBoardManager():
			return self.getMessage_NB('The Board has Expired.')	
			
		if self.isNeoBoardManager() and self._upgradeNeeded():
			return self._upgradeForm()
	        
		header = apply(self.getThemeObject_NB(id='NeoBoardHeader', obj_type='templates', render=0), (self, self.REQUEST))
		footer = apply(self.getThemeObject_NB(id='NeoBoardFooter', obj_type='templates', render=0), (self, self.REQUEST))
		
      		return header + footer

	manage_main = index_html

	def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
		"""
			Return index_html
		"""		
				
		return self.index_html()
				
	def _resetNeoBoard(self):
		"""
			Resets the board when articles get moved or deleted
		"""		

		articles =[]
		for art in self.objectValues('NeoBoard Article'):
			articles.append(art.id)
			
		num = len(articles)
			
		next_id = getNextElementName(articles, prefix='')

		if num > 0:	
			self.setNextArticleNum(int(next_id) - 1)
			self.setTotalNumArticles(num)
		else:
			self.setNextArticleNum(0)
			self.setTotalNumArticles(0)
  
  	def _reRenderAll(self):
		""" 
			Re-renders all renderable objects the container holds when renderable formats are changed
		"""			
		for article in self.getNeoBoardArticles():
			article.updateContent()
			article.reindex_object()

	def getCatalog(self):
		"""
			Returns ZCatalog
		"""
		try:
			return getattr(self.getSelf(), 'Catalog')
		except:
			return None

			
	def _addCatalog(self):
		"""
			Adds a ZCatalog for this board
		"""
		
		if hasattr(self.aq_base, 'Catalog'):
			self.manage_delObjects(['Catalog',])
			
		self.manage_addProduct['ZCatalog'].manage_addZCatalog('Catalog', self.getTitle() + ' - ' + 'Catalog')
		
		catalog = self._getOb('Catalog')
	
		catalog.addIndex(name='author', type='FieldIndex')
		catalog.addIndex(name='getRenderedContent', type='TextIndex')
		catalog.addIndex(name='getAttachmentsSearchableText', type='TextIndex')
		self._reRenderAll()

	def getMailHosts_NB(self):
		"""
			Returns the list of available mailhost objects
		"""	
		try:
			return self.superValues(('Mail Host',))
		except:
			return []
			
	def getMailHost_NB(self):
		""" 
			Returns the current mailhost instance
		"""
		
		return getattr(self, self.mailhost)


	def sendmail(self, from_addr, to_addr, reply_addr, subject, body, attachments=None, html=0):
		""" 
			Sends out emails
		"""	
	
		mailer = NB_Mailer(self.getMailHost(), from_addr, to_addr, reply_addr, subject, body, attachments, html)
		mailer.send()

	def pageCeil(self, num1, num2):
		""" 
			Mimics the PHP's ceil() function
		"""	
		
		quotient, remainder = divmod(int(num1), int(num2))
		
		if remainder > 0:
			return quotient + 1
		else:
			return quotient
	
	def _upgradeNeeded(self):
		if not hasattr(self, '_upgraded_up_to'):
			return 1
			
		if not NeoBoard.__NB_VERSION__ == self._upgraded_up_to:
			return 1
		else:
			return 0

	def _upgradeForm(self):

		form_str = "<html><body><div align=\"center\">"
		form_str = form_str + "<ul><li>Please update the board to date.</li></ul>"
		form_str = form_str + "<form action=\"" + self.absolute_url()+  "/selfUpgrade\" method=\"post\">\n"
		form_str = form_str + "<input type=\"submit\" name=\"submit\" value=\"Upgrade to version " + self.__NB_VERSION__ + "\">"
		form_str = form_str + "</form>\n"
		form_str = form_str + "</div></body></html>\n"
		
		return form_str
		
	security.declareProtected('Edit NeoBoard', 'selfUpgrade')
	def selfUpgrade(self):		
		"""
			Upgrade me
		"""

		if hasattr(self.aq_base, '_upgraded_up_to'):
			if self._upgraded_up_to == '1.0b':
				return 'Already Upgraded Up To: version ' + self._upgraded_up_to


		if not hasattr(self.aq_base, 'bad_extensions'):
			self.bad_extensions = ['vbs',]

		if not hasattr(self.aq_base, 'theme'):
			self.theme = 'default'

		if not hasattr(self.aq_base, 'formats'):
			self.formats = ('HTML', 'STX',)

		if not hasattr(self.aq_base, 'charset'):
			self.charset = 'ISO-8859-1'

		if not hasattr(self.aq_base, 'language'):
			self.language = 'en-US'
			
		if not hasattr(self.aq_base, 'effective_date'):
			self.effective_date= DateTime()

		if not hasattr(self.aq_base, 'creation_date'):
			self.creation_date= DateTime()

			
		if not hasattr(self.aq_base, 'expire_date'):
			self.expire_date=None
			
		if not hasattr(self.aq_base, 'cacheable_content'):			
			self.cacheable_content=1

		if not hasattr(self.aq_base, 'num_attachments'):						
			self.num_attachments=1

		if not hasattr(self.aq_base, 'acl_user_auth'):
			self.acl_user_auth=0
		
		if not hasattr(self.aq_base, 'allow_empty_body'):
			self.allow_empty_body=0

		if not hasattr(self.aq_base, 'allow_empty_password'):		
			self.allow_empty_password=0
			
		if not hasattr(self.aq_base, 'allow_empty_email'):					
			self.allow_empty_email=0

		if not hasattr(self.aq_base, 'auto_approve_article'):								
			self.auto_approve_article=1
						
		upgraded_objs = [self.absolute_url(),]
		for article in self.getNeoBoardArticles():
			if hasattr(article.aq_base, 'selfUpgrade'):
				article.selfUpgrade()
				upgraded_objs.append(article.absolute_url())
				
		self._addCatalog()
		
		self._upgraded_up_to = '1.0b'
		
		ret_str = "<html><body>Upgrade Complete.\n" + str(len(upgraded_objs)) + " object(s) updated.\n"
		ret_str = ret_str + "<div align=\"center\"><a href=\"" + self.absolute_url() +  "\">Click here to go back to the board.</a></div></body></html>"
		
		return ret_str
	
InitializeClass(NeoBoard)
